@@ -1,5 +1,11 @@
 # zzz.R - Package load and attach hooks
 
+#' @importFrom cli cli_alert_success cli_alert_info cli_alert_warning
+#' @importFrom cli cli_alert_danger cli_progress_step
+#' @importFrom cli cli_h1 cli_h2 cli_h3 cli_bullets cli_warn cli_abort
+#' @importFrom grDevices col2rgb as.raster
+#' @importFrom stats runif setNames
+#' @importFrom utils download.file unzip
 #' @keywords internal
 "_PACKAGE"
 
@@ -7,7 +13,7 @@
   # 1) Register knitr helpers only if knitr is installed
   if (requireNamespace("knitr", quietly = TRUE)) {
     tryCatch(
-      register_knitr_engine(),
+      register_cardargus_knitr(),
       error = function(e) {
         # Silently ignore to avoid breaking package load
       }
@@ -35,12 +41,3 @@
     })
   }
 }
-
-.onAttach <- function(libname, pkgname) {
-  packageStartupMessage(
-    "cardargus: Generate beautiful SVG cards with embedded styles\n",
-    "Use svg_card() to create cards\n",
-    "Use svg_to_png() or svg_to_png_chrome() to export to PNG\n",
-    "For best font support: setup_fonts()"
-  )
-}
\ No newline at end of file
